==> eaiautomatontools-1.0.4-py3-none-any.whl: eaiautomatontools/finders.py <==
# -*- coding: utf-8 -*-
import logging
from .drivers_tools import web_drivers_tuple
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.common.action_chains import ActionChains

log = logging.getLogger(__name__)


def find_element(driver=None, field=None):
    """
    Look up for the field described as a dictionary {"type": string, "value":}.
    Example: {"type": "id", "value": "frmCentreNumber"}
    Optionally, you can specify a text an call the find_from_elements method
    :param driver: a selenium web driver
    :param field: a dictionary
    :raise AssertionError: if driver is not a proper web driver instance or
            the field is not a dictionary
    :raise KeyError: If the field variable doesn't contain the expected keys i.e. type and value
    :raise NoSuchElementException: If the field doesn't exist
    :return: a selenium web element
    """
    try:
        assert driver is not None and isinstance(driver, web_drivers_tuple()),\
            "Driver is expected."
        assert isinstance(field, dict), "Field must be a dictionary"
        if "type" not in field.keys() or "value" not in field.keys():
            raise KeyError("The field argument doesn't contains either the 'type' or 'value' key.")
        if "text" in field.keys():
            return find_from_elements(driver=driver, field=field, text=field['text'])
        # Define the association between a type and a selenium find action
        switcher = {
            "id": driver.find_element_by_id,
            "name": driver.find_element_by_name,
            "class_name": driver.find_element_by_class_name,
            "css": driver.find_element_by_css_selector,
            "link_text": driver.find_element_by_link_text,
            "partial_link_text": driver.find_element_by_partial_link_text,
            "tag_name": driver.find_element_by_tag_name,
            "xpath": driver.find_element_by_xpath
        }
        element = switcher[field["type"]](field["value"])
        actions = ActionChains(driver)
        actions.move_to_element(element)
        actions.perform()
        return element
    except AssertionError as assertion:
        log.error("finders.find_element raised an assertion with following input"
                  f" driver:'{driver}' and field:'{field}'. Assertion is '{assertion.args}'")
        raise
    except KeyError as key_error:
        log.error(key_error.args[0])
        raise KeyError(key_error)
    except NoSuchElementException as no_such_element:
        log.error("In find_element didn't find the element '{}'."
                  " Exception is '{}'".format(field, no_such_element.args))
        raise NoSuchElementException("Element designed by field '{}'"
                                     " could not be located.".format(field)) from None


def find_elements(driver=None, field=None):
    """
    Look up for the field described as a dictionary {"type": string, "value":}.
    Example: {"type": "id", "value": "frmCentreNumber"}
    :param driver: a selenium web driver
    :param field: a dictionary
    :raise AssertionError: if driver is not a proper web driver instance or
            the field is not a dictionary
    :raise KeyError: If the field variable doesn't contain the expected keys i.e. type and value
    :return: a list of selenium web element
    """
    try:
        assert driver is not None and isinstance(driver, web_drivers_tuple()),\
            "Driver is expected."
        assert isinstance(field, dict), "Field must be a dictionary"
        if "type" not in field.keys() or "value" not in field.keys():
            raise KeyError("The field argument doesn't contains either the 'type' or 'value' key.")
        # Define the association between a type and a selenium find action
        switcher = {
            "id": driver.find_elements_by_id,
            "name": driver.find_elements_by_name,
            "class_name": driver.find_elements_by_class_name,
            "css": driver.find_elements_by_css_selector,
            "link_text": driver.find_elements_by_link_text,
            "partial_link_text": driver.find_elements_by_partial_link_text,
            "tag_name": driver.find_elements_by_tag_name,
            "xpath": driver.find_elements_by_xpath
        }
        return switcher[field["type"]](field["value"])
    except AssertionError as assertion:
        log.error("finders.find_elements raised an assertion with following input"
                  " driver:'{}', field:'{}'. Assertion is '{}'".format(driver, field,
                                                                       assertion.args))
        raise
    except KeyError as key_error:
        log.error(key_error.args[0])
        raise KeyError(key_error)


def find_from_elements(driver=None, field=None, text=None):
    """
    Try to locate an element using his text
    :param driver: a selenium web driver
    :param field: a dictionary
    :param text: a string
    :raise AssertionError: from the eaifinders.find_elements method
    :raise KeyError: from the eaifinders.find_elements method
    :raise NoSuchElementException: when no element is found
    :return: a selenium web element
    """
    elements = find_elements(driver=driver, field=field)
    return_element = None

    if text is None:
        raise AttributeError("text must be provided")
    if not isinstance(text, str):
        raise AttributeError("text must be a string")
    if not text:
        raise AttributeError("text must be non-empty")

    for element in elements:
        if element.text == text:
            log.debug(element.text)
            return_element = element
            break
        if element.get_attribute("value") == text:
            log.debug(element.get_attribute("value"))
            return_element = element
            break

    if return_element is None:
        raise NoSuchElementException(f"Element designed by field '{field}' and text '{text}'"
                                     " could not be located.")
    else:
        actions = ActionChains(driver)
        actions.move_to_element(return_element)
        actions.perform()
        return return_element


def find_sub_element_from_element(web_element=None, field=None):
    """
    Return a sub element from the element
    :param web_element:
    :param field:
    :return:
    """
    try:
        assert isinstance(web_element, WebElement), "web_element must be a WebElement object"
        assert isinstance(field, dict), "Field must be a dictionary"
        if "type" not in field.keys() or "value" not in field.keys():
            raise KeyError("The field argument doesn't contains either the 'type' or 'value' key.")

        switcher = {
            "id": web_element.find_element_by_id,
            "name": web_element.find_element_by_name,
            "class_name": web_element.find_element_by_class_name,
            "css": web_element.find_element_by_css_selector,
            "link_text": web_element.find_element_by_link_text,
            "partial_link_text": web_element.find_element_by_partial_link_text,
            "tag_name": web_element.find_element_by_tag_name,
            "xpath": web_element.find_element_by_xpath
            }

        element = switcher[field["type"]](field["value"])
        actions = ActionChains(web_element.parent)
        actions.move_to_element(element)
        actions.perform()
        return element
    except AssertionError as assertion:
        log.error("finders.find_sub_element_from_element raised an assertion with"
                  " following input\n web_element:'{}', field:'{}'. "
                  "Assertion is '{}'".format(web_element, field, assertion.args))
        raise
    except KeyError as key_error:
        log.error("In find_sub_element_from_element didn't find the method for '{}'"
                  " finder method".format(field["type"]))
        raise KeyError(key_error)

==> eaiautomatontools-1.0.5-py3-none-any.whl: eaiautomatontools/finders.py <==
# -*- coding: utf-8 -*-
import logging

from deprecated.classic import deprecated

from .drivers_tools import web_drivers_tuple
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.common.action_chains import ActionChains

log = logging.getLogger(__name__)


def __find_element(web_element, field: dict):
    switcher = {
        "id": web_element.find_element_by_id,
        "name": web_element.find_element_by_name,
        "class_name": web_element.find_element_by_class_name,
        "css": web_element.find_element_by_css_selector,
        "link_text": web_element.find_element_by_link_text,
        "partial_link_text": web_element.find_element_by_partial_link_text,
        "tag_name": web_element.find_element_by_tag_name,
        "xpath": web_element.find_element_by_xpath
    }
    return switcher[field["type"]](field["value"])


def __find_elements(web_element, field: dict):
    switcher = {
        "id": web_element.find_elements_by_id,
        "name": web_element.find_elements_by_name,
        "class_name": web_element.find_elements_by_class_name,
        "css": web_element.find_elements_by_css_selector,
        "link_text": web_element.find_elements_by_link_text,
        "partial_link_text": web_element.find_elements_by_partial_link_text,
        "tag_name": web_element.find_elements_by_tag_name,
        "xpath": web_element.find_elements_by_xpath
    }
    return switcher[field["type"]](field["value"])


def find_element(driver=None, field=None, web_element=None):
    """
    Look up for the field described as a dictionary {"type": string, "value":}.
    Example: {"type": "id", "value": "frmCentreNumber"}
    Optionally, you can specify a text an call the find_from_elements method
    :param driver: a selenium web driver
    :param field: a dictionary
    :param web_element: a web_element to search from
    :raise AssertionError: if driver is not a proper web driver instance or
            the field is not a dictionary
    :raise KeyError: If the field variable doesn't contain the expected keys i.e. type and value
    :raise NoSuchElementException: If the field doesn't exist
    :return: a selenium web element
    """
    try:
        if driver is None or not isinstance(driver, web_drivers_tuple()):
            raise AttributeError("Driver is expected")
        if not isinstance(field, dict):
            raise AttributeError(f"{field} is not a dictionary")
        if web_element is not None and not isinstance(web_element, WebElement):
            raise AttributeError("When provided web_element must be a WebElement"
                                 f"Get {type(web_element)}")
        if "type" not in field.keys() or "value" not in field.keys():
            raise KeyError("The field argument doesn't contains either the 'type' or 'value' key.")
        if "text" in field.keys():
            return find_from_elements(driver=driver, field=field, text=field['text'])
        # Define the association between a type and a selenium find action
        if web_element is None:
            element = __find_element(driver, field)
        else:
            element = __find_element(web_element, field)

        actions = ActionChains(driver)
        actions.move_to_element(element)
        actions.perform()
        return element
    except AttributeError as attribute_error:
        log.error("finders.find_element raised an assertion with following input"
                  f" driver:'{driver}' and field:'{field}' and web_element: '{web_element}'."
                  f" Assertion is '{attribute_error.args}'")
        raise AttributeError(attribute_error)
    except KeyError as key_error:
        log.error(key_error.args[0])
        raise KeyError(key_error)
    except NoSuchElementException as no_such_element:
        log.error("In find_element didn't find the element '{}'."
                  " Exception is '{}'".format(field, no_such_element.args))
        raise NoSuchElementException("Element designed by field '{}'"
                                     " could not be located.".format(field)) from None


def find_elements(driver=None, field=None, web_element=None):
    """
    Look up for the field described as a dictionary {"type": string, "value":}.
    Example: {"type": "id", "value": "frmCentreNumber"}
    :param web_element:
    :param driver: a selenium web driver
    :param field: a dictionary
    :raise AssertionError: if driver is not a proper web driver instance or
            the field is not a dictionary
    :raise KeyError: If the field variable doesn't contain the expected keys i.e. type and value
    :return: a list of selenium web element
    """
    try:
        if driver is None or not isinstance(driver, web_drivers_tuple()):
            raise AttributeError("Driver is expected")
        if not isinstance(field, dict):
            raise AttributeError(f"{field} is not a dictionary")
        if web_element is not None and not isinstance(web_element, WebElement):
            raise AttributeError("When provided web_element must be a WebElement"
                                 f"Get {type(web_element)}")
        if "type" not in field.keys() or "value" not in field.keys():
            raise KeyError("The field argument doesn't contains either the 'type' or 'value' key.")
        # Define the association between a type and a selenium find action
        if web_element is not None:
            return __find_elements(web_element, field)
        else:
            return __find_elements(driver, field)
    except AttributeError as attribute_error:
        log.error("finders.find_elements raised an assertion with following input"
                  f" driver:'{driver}' and field:'{field}' and web_element: '{web_element}'."
                  f" Assertion is '{attribute_error.args}'")
        raise AttributeError(attribute_error)
    except KeyError as key_error:
        log.error(key_error.args[0])
        raise KeyError(key_error)


def find_from_elements(driver=None, field=None, text=None, web_element=None):
    """
    Try to locate an element using his text
    :param web_element:
    :param driver: a selenium web driver
    :param field: a dictionary
    :param text: a string
    :raise AssertionError: from the eaifinders.find_elements method
    :raise KeyError: from the eaifinders.find_elements method
    :raise NoSuchElementException: when no element is found
    :return: a selenium web element
    """
    elements = find_elements(driver=driver, field=field, web_element=web_element)
    return_element = None

    if text is None:
        raise AttributeError("text must be provided")
    if not isinstance(text, str):
        raise AttributeError("text must be a string")
    if not text:
        raise AttributeError("text must be non-empty")

    for element in elements:
        if element.text == text:
            log.debug(element.text)
            return_element = element
            break
        if element.get_attribute("value") == text:
            log.debug(element.get_attribute("value"))
            return_element = element
            break

    if return_element is None:
        raise NoSuchElementException(f"Element designed by field '{field}' and text '{text}'"
                                     " could not be located.")
    actions = ActionChains(driver)
    actions.move_to_element(return_element)
    actions.perform()
    return return_element


@deprecated(version="1.0.5", reason="You should user find_element with a web_element")
def find_sub_element_from_element(web_element=None, field=None):
    """
    Return a sub element from the element
    :param web_element:
    :param field:
    :return:
    """
    try:
        assert isinstance(web_element, WebElement), "web_element must be a WebElement object"
        assert isinstance(field, dict), "Field must be a dictionary"
        if "type" not in field.keys() or "value" not in field.keys():
            raise KeyError("The field argument doesn't contains either the 'type' or 'value' key.")

        switcher = {
            "id": web_element.find_element_by_id,
            "name": web_element.find_element_by_name,
            "class_name": web_element.find_element_by_class_name,
            "css": web_element.find_element_by_css_selector,
            "link_text": web_element.find_element_by_link_text,
            "partial_link_text": web_element.find_element_by_partial_link_text,
            "tag_name": web_element.find_element_by_tag_name,
            "xpath": web_element.find_element_by_xpath
            }

        element = switcher[field["type"]](field["value"])
        actions = ActionChains(web_element.parent)
        actions.move_to_element(element)
        actions.perform()
        return element
    except AssertionError as assertion:
        log.error("finders.find_sub_element_from_element raised an assertion with"
                  " following input\n web_element:'{}', field:'{}'. "
                  "Assertion is '{}'".format(web_element, field, assertion.args))
        raise
    except KeyError as key_error:
        log.error("In find_sub_element_from_element didn't find the method for '{}'"
                  " finder method".format(field["type"]))
        raise KeyError(key_error)
